;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; GNU GENERAL PUBLIC LICENSE ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; Interactive model for studying the effects of quality uncertainty in markets
;; Copyright (C) 2014  
;; María Pereda, David Poza, José I. Santos, José M. Galán
;; 
;; This program is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.

;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


;-----------------------------
;           Breeds
;-----------------------------

breed [buyers buyer]

;-----------------------------
;     Variables
;-----------------------------

buyers-own[
  ID
  ;nickname
  price
  last-price
  unit-bought?
  quality
  last-quality
  current-surplus
  total-surplus
  my-neighbors
]


globals [
  nround
  num-buyers
  sellers-marginalcost ;; sellers' marginal cost ordered from low to high
  buyers-list ;; buyers ordered by buyers' resevation prices
  buyers-in-round-list
  clearing-price ;; market price 
  traded-units
]

;-----------------------------
;    Start/Reset Server
;-----------------------------

to start/reset-server
  clear-output
  hubnet-set-client-interface "COMPUTER" []
  hubnet-reset
  ca
  output-print "Waiting for users to connect..."
  output-print "Please, click on LISTEN button"
  output-print "When all the users are connected, ..."
  output-print "    (1) click on SETUP TRADE button to set up a trade sesion ..."
  output-print "and (2) click on GO TRADE button to start the sesion."
  ;setup
end

;-----------------------------
;     Listen Clients
;-----------------------------

to listen
  
    while [hubnet-message-waiting?]
    [
      hubnet-fetch-message
      ifelse hubnet-enter-message?
      [
        create-new-buyer
        output-print (word "The user " hubnet-message-source " has logged in the system.")
      ]
      [
        ifelse hubnet-exit-message?
        [
          remove-buyer
        ]
        [
          ask buyers with [ID = hubnet-message-source]
          [
            execute-command hubnet-message-tag
          ]
        ]
      ]
    ]
    
    every 0.1 [display]

end

;-----------------------------
;     Setup
;-----------------------------

to setup
    setup-variables
    reset-timer

;    ;; Check if num-neighbors is correct
    if ( (nround = 0) AND (network?) )[
      let max-num-neighbors ( (floor (num-buyers - 1) / 2) * 2)
      if (num-neighbors > max-num-neighbors) 
       [ set num-neighbors max-num-neighbors
         user-message (word "The number of neighbors should be an even number not greater than " max-num-neighbors ".")
       ]
    ]
   
   clear-all-plots
   setup-plots;; setup commands are integrated witin plots in GUI
   
   layout-circle buyers-list (max-pxcor - 10)
   if (network?)[create-random-network]
   
   setup-control-panels
   update-instructor-view
   wait 5
   update-client-view
  
end

to setup-variables
   set nround 0 
   set num-buyers count buyers
   set clearing-price 0
   ; create offer function
   set sellers-marginalcost n-values num-buyers [y0 + a * ?] ; supply from lower to higher marginal costs
   set buyers-list shuffle sort buyers ; initial random list of buyers used to create the random network
   ask buyers [
    set price 0
    set last-price 0
    set quality 0
    set last-quality 0
    set current-surplus 0
    set total-surplus 0 
    set unit-bought? false
    set my-neighbors no-turtles
  ]
end


;-----------------------------
;     Main
;----------------------------- 

to go
    
    ; (1) Start trading
    reset-timer
    set nround nround + 1
    start-trading
    ;---------- Waiting for buyer reservation prices --------
    while [timer < trading-time]
      [listen]
        
    ; (2) Clear market: calculate price and traded units
    clear-market
        
    ; (3) Update surplus
    update-surplus

    ; update plots and instructor and clients' view
    do-plotting
    update-instructor-view
    update-client-view
           
end


;-----------------------------
;        Procedures
;-----------------------------



to start-trading
  ; create offer function
  set sellers-marginalcost n-values num-buyers [y0 + a * ?] ; supply from lower to higher marginal costs
  
  output-print " "
  output-print (word "ROUND " nround ". New round starts!")
  output-print "It's time for buyers to bid price."
  ask buyers [ 
    set price 0
    set unit-bought? false     
    set current-surplus 0
    set quality 0
    hubnet-send ID "Round" nround
    hubnet-broadcast "Budget" budget
    hubnet-broadcast "Price" 0
    hubnet-send ID "Market price" "trading"
    hubnet-send ID "Purchase?" "trading"
    hubnet-send ID "Item quality" "none"
    hubnet-send ID "Surplus" budget
    hubnet-send ID "Information" (word "Please bid price, you have " trading-time "seconds to make your decision") ]
end  

to clear-market  
  output-print "Clearing the market."
  set buyers-list sort-on [price] buyers  
  set buyers-list reverse buyers-list ;sorted demand from higher to lower prices
  
  let num-sellers num-buyers   
  set buyers-in-round-list []
  set traded-units 0
  ask buyers [ set unit-bought? false ]
  
  ;; calculate the number of traded units, and create the lists of buyers in the round
  while [traded-units < num-buyers and [price] of item traded-units buyers-list >= item traded-units sellers-marginalcost ]
  [
     set buyers-in-round-list lput (item traded-units buyers-list) buyers-in-round-list
     set traded-units traded-units + 1 
  ]  
  ;; if there's going to be any trade, calculate the price.
  ;; This price-setting formula takes into account the satisfied supply and demand 
  ;; and the pressure of the unsatisfied supply and demand 
  ifelse (traded-units > 0) [
    let min-price ifelse-value (num-sellers = traded-units) 
                  [ item (traded-units - 1) sellers-marginalcost ]
                  [ min list ([price] of item (traded-units - 1) buyers-list)
                             ( item (traded-units) sellers-marginalcost ) ]
    let max-price ifelse-value (num-buyers = traded-units)
                  [ item (traded-units - 1) sellers-marginalcost ]
                  [ max list ( item (traded-units - 1) sellers-marginalcost)
                             ([price] of item (traded-units) buyers-list) ]    
    set clearing-price (min-price + max-price) / 2 
  ]
  [ set clearing-price 0 ]
  output-print (word traded-units "  units have been traded at the price " clearing-price )
end

to update-surplus
  output-print "Updating buyers' surplus."
  foreach buyers-list [
    ifelse member? ? buyers-in-round-list 
      [  ; the buyer bought an item
        ask ? [ 
          set quality max (list 0 random-normal quality-mean quality-sd) ;;lower trimmed normal
          set last-quality quality
          set last-price price
          set current-surplus budget + quality - clearing-price
          set total-surplus ( total-surplus + current-surplus )
          set unit-bought? true ]
      ]
      [  ; the buyer didn't buy an item
        ask ? [ 
          set last-price price
          set current-surplus budget
          set total-surplus ( total-surplus + current-surplus )
          set unit-bought? false ]
      ]
    
    ask ? [
          hubnet-send ID "Market price" precision clearing-price 2
          hubnet-send ID "Purchase?" unit-bought?
          hubnet-send ID "Item quality" precision quality 2
          hubnet-send ID "Surplus" precision current-surplus 2
          hubnet-send ID "Total surplus" precision total-surplus 2
          hubnet-send ID "Information" ifelse-value unit-bought? ["Yeah!! You have bought an item!"] ["You have not bought an item"]
          if (network? and any? my-neighbors)
            [ hubnet-send ID "Item quality of other buyers" [precision quality 2] of my-neighbors ] ]
  ]

  
  wait 4
  output-print (word "Results checking time. Round " (nround + 1) " will start in " after-trading-time " seconds.")
  hubnet-broadcast "Information" (word "Check results. A new round will start in " after-trading-time " seconds.")  
  wait after-trading-time - 4
  
end
  


to create-new-buyer
  create-buyers 1
  [
    set ID hubnet-message-source
    ;set nickname hubnet-message-source 
    set size 5
    set shape "person"
    hubnet-send ID "Your ID" ID
    hubnet-send ID "Information" "You have successfully logged in. Please wait for other users to connect."   
  ]    
end
      
to remove-buyer
  ask buyers with [ID = hubnet-message-source]
  [
    die
  ]
end

to execute-command [command]
    if command = "Price" and timer < trading-time 
    [
      ifelse (hubnet-message > budget) [
        hubnet-send hubnet-message-source "Information" "ALERT! Your reservation price must be lower than your surplus"
        hubnet-send hubnet-message-source "Price" "Error"]
      [ hubnet-send hubnet-message-source "Information" "Your reservation price has been registered."
        set price hubnet-message]
      ]
end

to create-random-network

  ask links [die]
  foreach buyers-list [
    let i-buyer ?
    let i-index-buyer position i-buyer buyers-list
    foreach n-values (floor (num-neighbors / 2)) [? + 1 ] [
       ask i-buyer [ create-link-with item ( (i-index-buyer + ?) mod num-buyers ) buyers-list ] ]
  ]
  ask buyers [ set my-neighbors link-neighbors ]   
end


;-----------------------------
;        Plots and World View
;-----------------------------

to do-plotting

  set-current-plot "market"
  ; plot supply
  set-current-plot-pen "supply" 
  plot-pen-reset
  foreach sellers-marginalcost [ plot ? ]
  ; plot demand
  set-current-plot-pen "demand" 
  plot-pen-reset
  foreach buyers-list [ plot [price] of ? ]

  set-current-plot "Price time serie"
  set-current-plot-pen "price" 
  plotxy nround clearing-price

  set-current-plot "Traded units time serie"
  set-current-plot-pen "units" 
  plotxy nround traded-units

end

to update-instructor-view

  ; labels
  if (labels = "none")  [ ask buyers [set label ""] ]
  if (labels = "last price")  [ ask buyers [set label (precision last-price 1)] ]
  if (labels = "last quality")  [ ask buyers [set label (precision last-quality 1)] ]

  ; colors
  ask buyers [ 
    ifelse unit-bought? 
      [ set color green]
      [ set color red ]
  ]
end

to update-client-view

  ask buyers [
    hubnet-clear-overrides ID 
    ; hide all buyers and links
    hubnet-send-override ID buyers "hidden?" [true]
    hubnet-send-override ID links "hidden?" [true]
    
    ; update color and labels of my neighbors and me
    hubnet-send-override ID self "color" [blue]
    hubnet-send-override ID self "label" [precision last-quality 2]
    hubnet-send-override ID my-neighbors "color" [orange]
    hubnet-send-override ID my-neighbors "label" [precision last-quality 2]
    hubnet-send-override ID self "hidden?" [false]
    hubnet-send-override ID my-neighbors "hidden?" [false]
    hubnet-send-override ID my-links "hidden?" [false]
 
  ]
end

to setup-control-panels
  output-print "Setting up a new trade session"
  ask buyers [
    hubnet-send ID "Round" 0
    hubnet-send ID "Information" "Setting up a new trade session"
    hubnet-send ID "Budget" budget
    hubnet-send ID "Price" 0
    hubnet-send ID "Market price" 0
    hubnet-send ID "Purchase?" false
    hubnet-send ID "Item quality" 0
    hubnet-send ID "Surplus" 0
    hubnet-send ID "Total surplus" 0
    hubnet-send ID "Item quality of other buyers" "" 
  ]
end
  
  
@#$#@#$#@
GRAPHICS-WINDOW
707
222
1081
617
45
45
4.0
1
10
1
1
1
0
0
0
1
-45
45
-45
45
0
0
1
tick
30.0

INPUTBOX
24
332
119
392
quality-mean
4
1
0
Number

TEXTBOX
22
187
246
205
Supply function y=y0+a*x\n\n
15
0.0
1

INPUTBOX
121
332
216
392
quality-sd
2
1
0
Number

INPUTBOX
575
64
670
124
trading-time
45
1
0
Number

TEXTBOX
575
39
736
58
Parameters
15
0.0
1

SWITCH
25
496
181
529
network?
network?
0
1
-1000

OUTPUT
265
183
697
327
12

TEXTBOX
265
155
360
175
Information
16
0.0
1

MONITOR
589
335
673
380
round
nround
0
1
11

MONITOR
499
335
580
380
num-buyers
count buyers
0
1
11

BUTTON
19
48
176
81
START/RESET SERVER
start/reset-server
NIL
1
T
OBSERVER
NIL
S
NIL
NIL
1

BUTTON
19
89
176
122
LISTEN
listen
T
1
T
OBSERVER
NIL
L
NIL
NIL
1

BUTTON
406
93
558
126
GO TRADE / PAUSE
go
T
1
T
OBSERVER
NIL
G
NIL
NIL
1

PLOT
499
439
687
615
Market
quantity
price
0.0
1.0
0.0
1.0
true
false
"set-plot-x-range 0 num-buyers\nset-plot-y-range 0 max sellers-marginalcost" ""
PENS
"supply" 1.0 0 -14070903 true "" ""
"demand" 1.0 0 -15040220 true "" ""

INPUTBOX
672
64
772
124
after-trading-time
30
1
0
Number

TEXTBOX
23
285
245
342
Product Quality \n~ N(quality-mean, quality-sd)
15
0.0
1

INPUTBOX
23
213
90
273
y0
2
1
0
Number

INPUTBOX
94
213
163
273
a
4
1
0
Number

TEXTBOX
24
406
174
444
Buyers\n\n
15
0.0
1

INPUTBOX
25
432
180
492
budget
10
1
0
Number

TEXTBOX
270
31
541
54
INSTRUCTOR CONTROL PANEL
18
0.0
1

SLIDER
24
538
182
571
num-neighbors
num-neighbors
0
10
2
2
1
NIL
HORIZONTAL

BUTTON
269
93
399
126
SETUP TRADE
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

MONITOR
499
387
583
432
traded units
traded-units
2
1
11

MONITOR
590
387
674
432
market price
clearing-price
2
1
11

PLOT
267
358
484
482
Price time serie
time
p
0.0
10.0
0.0
10.0
true
false
"" ""
PENS
"price" 1.0 0 -16777216 true "" ""

PLOT
266
490
484
614
Traded units time serie
time
units
0.0
10.0
0.0
10.0
true
false
"" ""
PENS
"units" 1.0 0 -16777216 true "" ""

CHOOSER
706
172
820
217
labels
labels
"none" "last price" "last quality"
0

TEXTBOX
22
158
202
182
Market Parameters
16
0.0
1

TEXTBOX
249
25
264
625
'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n
12
4.0
1

TEXTBOX
271
69
421
88
Sesion
16
0.0
1

TEXTBOX
271
336
421
354
Variables and Plots
15
0.0
1

TEXTBOX
23
25
173
44
Server
16
0.0
1

TEXTBOX
14
134
1097
152
-  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  - -  -  -  -  -  -  -  -  - -  -  -  -  -  -  -  -  - \n
11
4.0
1

TEXTBOX
7
25
22
625
'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n
12
4.0
1

TEXTBOX
9
10
1087
28
-  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  - -  -  -  -  -  -  -  -  -  -  -  -  -  -  - \n
11
4.0
1

TEXTBOX
1092
16
1107
631
'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n'\n
12
4.0
1

TEXTBOX
8
617
1094
635
-  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  - -  -  -  -  -  -  -  -  -  -  -  -  -  -  - \n
11
4.0
1

@#$#@#$#@
## QUICK GUIDE:

An installation, configuration and user manual can be found at:
https://www.openabm.org/model/4214/
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
15
Circle -1 true true 203 65 88
Circle -1 true true 70 65 162
Circle -1 true true 150 105 120
Polygon -7500403 true false 218 120 240 165 255 165 278 120
Circle -7500403 true false 214 72 67
Rectangle -1 true true 164 223 179 298
Polygon -1 true true 45 285 30 285 30 240 15 195 45 210
Circle -1 true true 3 83 150
Rectangle -1 true true 65 221 80 296
Polygon -1 true true 195 285 210 285 210 240 240 210 195 210
Polygon -7500403 true false 276 85 285 105 302 99 294 83
Polygon -7500403 true false 219 85 210 105 193 99 201 83

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

wolf
false
0
Polygon -16777216 true false 253 133 245 131 245 133
Polygon -7500403 true true 2 194 13 197 30 191 38 193 38 205 20 226 20 257 27 265 38 266 40 260 31 253 31 230 60 206 68 198 75 209 66 228 65 243 82 261 84 268 100 267 103 261 77 239 79 231 100 207 98 196 119 201 143 202 160 195 166 210 172 213 173 238 167 251 160 248 154 265 169 264 178 247 186 240 198 260 200 271 217 271 219 262 207 258 195 230 192 198 210 184 227 164 242 144 259 145 284 151 277 141 293 140 299 134 297 127 273 119 270 105
Polygon -7500403 true true -1 195 14 180 36 166 40 153 53 140 82 131 134 133 159 126 188 115 227 108 236 102 238 98 268 86 269 92 281 87 269 103 269 113

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 5.0.5
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
MONITOR
10
44
192
93
Your ID
NIL
3
1

MONITOR
72
132
513
181
Information
NIL
3
1

MONITOR
8
131
65
180
Round
NIL
3
1

INPUTBOX
111
229
204
289
Price
0
1
0
Number

TEXTBOX
113
190
229
228
Max. purchase price
13
0.0
1

TEXTBOX
17
293
99
311
Results
14
0.0
1

MONITOR
114
434
207
483
Surplus
NIL
2
1

MONITOR
113
375
206
424
Item quality
NIL
2
1

MONITOR
115
493
208
542
Total surplus
NIL
2
1

MONITOR
11
552
517
601
Item quality of other buyers
NIL
2
1

TEXTBOX
11
106
224
127
Trade session information
14
0.0
1

TEXTBOX
257
221
520
239
Me (blue) and other buyers (orange)
13
0.0
1

TEXTBOX
219
588
369
606
NIL
5
0.0
1

TEXTBOX
10
10
221
36
Buyer Control Panel
18
0.0
1

MONITOR
10
317
104
366
Purchase?
NIL
0
1

MONITOR
113
317
206
366
Market price
NIL
2
1

MONITOR
10
229
103
278
Budget
NIL
3
1

TEXTBOX
15
422
107
496
Surplus =  +Budget +Item quality  -Market Price
13
0.0
1

VIEW
216
241
516
541
0
0
0
1
1
1
1
1
0
1
1
1
-45
45
-45
45

@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
0
@#$#@#$#@
